@@ -10,13 +10,16 @@ from time import sleep
 from numerous_image_tools.errors import tb_str
 from numerous.client import ScenarioStatus, NumerousClient
 from numerous.client.data_source import DataSourceStreamStarting
+import networkx as nx
+from typing import List
 
 logger = logging.getLogger('sim-setup')
 
 class Component:
-    def __init__(self, name: str = None, folder: str = None, path: str = None, handle: str = None,
+    def __init__(self, component_id:str, container_id: str, name: str = None, folder: str = None, path: str = None, handle: str = None,
                  constants: dict = None, parameters: dict = None,
                  import_flag = False):
+
         self._folder = Path(folder)
         self._path = Path(path)
         self._handle = handle
@@ -26,15 +29,24 @@ class Component:
         self._entrypoint = None
         self.item = None
         self.name = name
+        self.component_id = component_id
+        self.container_id = container_id
         self._import = import_flag
         self.inputs = {}
         self.model = None
         self.components = {}
 
-    def _initialize(self, system):
+    def _initialize(self, system, wrapper=None):
         if not self._import:
             return
-        item = self._entrypoint(self.name, system)
+
+        if wrapper:
+            item = wrapper(self, self.name, system)
+        else:
+            item = self._entrypoint(self.name, system)
+
+
+
         self.item = item
         return item
 
@@ -64,11 +76,49 @@ class Project:
                                    for project_parameter in project_parameters}
 
 
+def unroll_components(components, graph, references, container_references, unrolled_components):
+    for component in components:
+        unroll_components(component.components.values(), graph, references, container_references, unrolled_components)
+
+        graph.add_node(component.component_id, component=component)
+        references.append(component.component_id)
+        if component.container_id:
+            container_references[component.container_id] = component.component_id
+
+        unrolled_components.append(component)
+
+def sort_components(components: List[Component]):
+    graph = nx.DiGraph()
+    references = []
+    container_references = {}
+
+    unrolled_components = []
+    unroll_components(components, graph, references, container_references, unrolled_components)
+
+
+
+    for component in unrolled_components:
+        for param_name, param in component.constants.items():
+            if isinstance(param, dict) and 'ref' in param:
+                ref = param['ref']
+                if ref in container_references:
+                    ref = container_references[ref]
+                    param['ref'] = ref
+                if ref in references:
+                    graph.add_edge(ref, component.component_id)
+                else:
+                    print('Warning: ', ref, ' is not defined. Referenced from ', component.component_id)
+
+
+    components_ = nx.get_node_attributes(graph, "component")
+    components_top_sorted =  [components_[n] for n in nx.topological_sort(graph)]
+    return components_top_sorted
+
 class NumerousSystem:
     def __init__(self, nc: NumerousClient=None, scenario=None, files=None, start_time=None, end_time=None,
                  model_folder=None, numerous_job=None,
-                 states=None, t=None, dt=None, input=None):
-
+                 states=None, t=None, dt=None, input=None, wrapper=None):
+        self.references = {}
         self.nc = nc
         self.scenario = scenario
         self.files = files
@@ -78,12 +128,14 @@ class NumerousSystem:
         self.logger = logging.getLogger('NumerousSystem')
         self._model_folder = model_folder
         self._has_model = False
+        self.wrapper = wrapper
 
         # Update status - clear current data
         self.nc.set_scenario_progress('Environment Initializing',ScenarioStatus.ENVIRONMENT_INITIALIZING, 0.0, clean=True)
 
         # Initialize other system variables
-        self.component_list, self.components = self._format_job_spec(self.scenario)
+        self.component_list, self.components = self._format_job_spec(self.scenario, model_folder)
+        self.unrolled_components = sort_components(self.components.values())
 
         # Download needed files
         self._download_custom_files()
@@ -137,11 +189,37 @@ class NumerousSystem:
     def initialize_model(self):
         self.numerous_job.system = self
 
-        for component in self.components.values():
-            component.model = component._initialize(self)
+        for component in self.unrolled_components:
+            for constant in component.constants.keys():
+
+                val = component.constants[constant]
+                #Handle references
+                if isinstance(val, dict) and 'ref' in val:
+
+                    if val['ref'] in self.references:
+                        ref_val = self.references[val['ref']]
+                    else:
+                        #TODO raise proper warning here
+                        ref_val = None
+                        print('Warning ref not found!')
+                        print(component.name)
+                        print('param: ', constant)
+                        print('ref: ', val['ref'])
+                        print('refs avail: ', self.references.keys())
+
+                    component.constants[constant] = ref_val
+
+            component.model = component._initialize(self, self.wrapper)
+
+            #Add newly create component model to references to be passed in for subsequent ref values
+
+            self.references[component.component_id] = component.model
+            if component.container_id:
+                self.references[component.container_id] = component.model
+
 
     def _dynamic_import_components(self):
-        for component in self.components.values():
+        for component in self.unrolled_components:
             self._add_path_topdown(component._folder)
             component._dynamicimport()
 
@@ -151,6 +229,8 @@ class NumerousSystem:
         component_data = copy.deepcopy(data)
         component_data.pop('_index')
         component_data.pop('_index_relative')
+        self.input_data = component_data
+
         for input_tag, value in component_data.items():
             try:
                 component_name = input_tag.split(".")[0]
@@ -162,11 +242,12 @@ class NumerousSystem:
             if component_name in self.components:
                 self.components[component_name].inputs.update({".".join(subcomponents): value})
 
-    def _format_job_spec(self, scenario):
+    def _format_job_spec(self, scenario, model_folder_name="."):
 
         simcomponents = scenario['simComponents']
         component_list = []
         components = {}
+        references = {}
 
         for simcomponent in simcomponents:
             if simcomponent.get('disabled', False) or simcomponent.get('isSubcomponent'):
@@ -206,23 +287,47 @@ class NumerousSystem:
         handle = None
         has_model = False
 
+        #references[simcomponent['id']] = None
+        if not simcomponent['disabled']:
+            # Get values for helper variables
+            item_class = simcomponent['item_class']
+            items = item_class.split(".")
+            component_name = simcomponent['name']
+            component_id = simcomponent['id']
+            container_id = simcomponent['containerID'] if 'containerID' in simcomponent else None
 
-        # Get values for helper variables
-        item_class = simcomponent['item_class']
-        items = item_class.split(".")
-        component_name = simcomponent['name']
+            if item_class and len(items) < 2:
+                raise IndexError(f"item {component_name} must have class of format x.y or x.y.z")
 
-        if item_class and len(items) == 3:
-            model_name = items[0]
-            model_file_name = items[1]
-            handle = items[2]
-            has_model = True
+            folder = "."
+            path = "."
 
-        # Format constant values ('parameters' in frontend)
-        component_constants = {
-            d['id']: d['value']
-            for d in simcomponent['parameters']
-        } if simcomponent['parameters'] else {}
+            if item_class:
+                has_model = True
+
+                model_file_name = items[-2]
+                handle = items[-1]
+
+                if len(items) > 2:
+                    model_name = items[0]
+
+                    folder = f"{self._model_folder}/{model_name}"
+                    path = f"{self._model_folder}.{model_name}.{model_file_name}"
+                else:
+                    folder = f"{self._model_folder}"
+                    path = f"{self._model_folder}.{model_file_name}"
+
+
+            component_constants = {}
+            if simcomponent['parameters']:
+                for d in simcomponent['parameters']:
+
+                    if d['type'] == 'file':
+                        component_constants[d['id']] = d['value']['name']
+                    elif d['type'] == 'reference':
+                        component_constants[d['id']] = {'ref': d['value']}
+                    else:
+                        component_constants[d['id']] = d['value']
 
         # Format parameters ('inputVariables' in frontend)
         component_parameters = {
@@ -232,10 +337,15 @@ class NumerousSystem:
 
         # Make a complete dict with formatted specifications
 
-        component = Component(name=component_name,
-                              folder=f"{self._model_folder}/{model_name}",
-                              path=f"{self._model_folder}.{model_name}.{model_file_name}", handle=handle,
+        component = Component(
+                              component_id=component_id,
+                              container_id=container_id,
+                              name=component_name,
+                              folder=folder,
+                              path=path, handle=handle,
                               constants=component_constants,
-                              parameters=component_parameters, import_flag=has_model and self._model_folder)
+                              parameters=component_parameters, import_flag=bool(item_class))
+        #component_list.append(component)
+        #components.update({component_name: component})
 
         return component
